@@ -17,7 +17,7 @@ __all__ = ['AppApi', 'UserApi', 'AccountApi', 'CheckoutApi', 'PreapprovalApi',
 # default is full access
 DEFAULT_SCOPE = getattr(
     settings, 'WEPAY_DEFAULT_SCOPE', "manage_accounts,collect_payments,"
-    "view_balance,view_user,preapprove_payments,send_money")
+    "view_user,preapprove_payments,manage_subscriptions,send_money")
 
 DEFAULT_MODELS = (
     ('app', 'djwepay.App'),
@@ -47,7 +47,7 @@ def get_wepay_model(obj_name):
 
 def is_abstract(obj_name):
     return not get_wepay_model_name(obj_name) == dict(DEFAULT_MODELS).get(obj_name)
-    
+
 
 class WePayLazy(LazyObject):
     def _setup(self):
@@ -67,24 +67,29 @@ class Api(object):
     app = AppLazy()
 
     def instance_update(self, response, commit=True):
-        previous_state = getattr(self, 'state', '') # app object doesn't have state?
+        is_new = False
+        previous_state = getattr(self, 'state', '')
         new_state = response.get('state', '')
         for key, value in response.iteritems():
+            if value == 0 and key.endswith('_id'):
+                value = None
             setattr(self, key, value)
         if new_state and new_state != previous_state:
             # using cache we eliminate duplicate calls to state_changed,
             # which has a chance of happening in multithreaded environment
             cache_key = "wepay-state-changed-%s-%s" % (type(self).__name__, self.pk)
             added = cache.add(cache_key, new_state)
+            is_new = True
             if not added:
                 stored_state = cache.get(cache_key)
                 if stored_state == new_state:
-                    return response
-            cache.set(cache_key, new_state)                
-            state_changed.send(sender=type(self), instance=self, 
-                               previous_state=previous_state)
+                    is_new = False
+                cache.set(cache_key, new_state)
         if commit:
             self.save()
+        if is_new:
+            state_changed.send(sender=type(self), instance=self,
+                               previous_state=previous_state)
         return self
 
     def instance_update_nocommit(self, response):
@@ -95,7 +100,7 @@ class Api(object):
 
     def get_callback_uri(self, **kwargs):
         return reverse('wepay:ipn', kwargs=kwargs)
-    
+
     def api_batch_create(self, **kwargs):
         return self.api.batch.create(client_id=self.app.client_id,
                                      client_secret=self.app.client_secret, **kwargs)
@@ -106,15 +111,21 @@ class AppApi(Api):
     def api_app(self, **kwargs):
         return self.api.app(
             client_id=self.client_id,
-            client_secret=self.client_secret, 
+            client_secret=self.client_secret,
             callback=self.instance_update, **kwargs)
 
     def api_app_modify(self, **kwargs):
         return self.api.app.modify(
             client_id=self.client_id,
-            client_secret=self.client_secret, 
+            client_secret=self.client_secret,
             callback=self.instance_update, **kwargs)
 
+    def api_account(self, **kwargs):
+        return self.api.account(
+            account_id=self.account.pk,
+            callback=self.account.instance_update, **kwargs)
+
+
     def api_oauth2_authorize(self, redirect_uri, **kwargs):
         return self.api.oauth2.authorize(
             self.client_id, redirect_uri, DEFAULT_SCOPE, **kwargs)
@@ -140,10 +151,10 @@ class AppApi(Api):
     def api_preapproval_create(self, **kwargs):
         Preapproval = get_wepay_model('preapproval')
         return self.api.preapproval.create(
-            client_id=self.client_id, 
+            client_id=self.client_id,
             client_secret=self.client_secret,
             callback_uri = self.get_callback_uri(obj_name='preapproval'),
-            callback=curry(Preapproval.objects.create_from_response, None), 
+            callback=curry(Preapproval.objects.create_from_response, self),
             **kwargs)
 
     def api_preapproval_find(self, **kwargs):
@@ -152,7 +163,7 @@ class AppApi(Api):
     def api_credit_card_create(self, **kwargs):
         CreditCard = get_wepay_model('credit_card')
         return self.api.credit_card.create(
-            client_id=self.client_id, 
+            client_id=self.client_id,
             callback=CreditCard.objects.create_from_response, **kwargs)
 
     def api_credit_card_find(self, **kwargs):
@@ -170,7 +181,7 @@ class UserApi(Api):
 
     def api_user(self, **kwargs):
         return self.api.user(
-            access_token=self.access_token, 
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_user_modify(self, **kwargs):
@@ -180,7 +191,6 @@ class UserApi(Api):
 
     def api_user_resend_confirmation(self, **kwargs):
         return self.api.user.resend_confirmation(
-            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_account_create(self, **kwargs):
@@ -188,7 +198,7 @@ class UserApi(Api):
         return self.api.account.create(
             access_token=self.access_token,
             callback_uri=self.get_callback_uri(
-                obj_name='account', user_id=self.user_id),
+                obj_name='account', user_id=self.pk),
             callback=curry(Account.objects.create_from_response, self), **kwargs)
 
     def api_account_find(self, **kwargs):
@@ -199,7 +209,9 @@ class AccountApi(Api):
 
     @cached_property
     def access_token(self):
-        return self.user.access_token
+        if self.user:
+            return self.user.access_token
+        return None
 
     @cached_property
     def uri(self):
@@ -208,8 +220,8 @@ class AccountApi(Api):
     def api_account(self, **kwargs):
         try:
             return self.api.account(
-                account_id=self.pk, 
-                access_token=self.access_token, 
+                account_id=self.pk,
+                access_token=self.access_token,
                 callback=self.instance_update, **kwargs)
         except WePayError, e:
             if e.code == 3003: # The account has been deleted
@@ -219,26 +231,26 @@ class AccountApi(Api):
 
     def api_account_modify(self, **kwargs):
         return self.api.account.modify(
-            account_id=self.pk, 
-            access_token=self.access_token, 
+            account_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
-        
+
     def api_account_delete(self, **kwargs):
         return self.api.account.delete(
-            account_id=self.pk, 
-            access_token=self.access_token, 
+            account_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_account_get_update_uri(self, **kwargs):
         return self.api.account.get_update_uri(
-            account_id=self.pk, 
-            access_token=self.access_token, 
+            account_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update_nocommit, **kwargs)
-        
+
     def api_account_get_reserve_details(self, **kwargs):
         return self.api.account.getreserve_details(
-            account_id=self.pk, 
-            access_token=self.access_token, 
+            account_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_identity, **kwargs)
 
     def api_checkout_create(self, **kwargs):
@@ -247,8 +259,8 @@ class AccountApi(Api):
             account_id=self.pk,
             access_token=self.access_token,
             callback_uri=self.get_callback_uri(
-                obj_name='checkout', user_id=self.user.pk), 
-            callback=curry(Checkout.objects.create_from_response, self), 
+                obj_name='checkout', user_id=self.user.pk),
+            callback=curry(Checkout.objects.create_from_response, self),
             **kwargs)
 
     def api_preapproval_create(self, **kwargs):
@@ -257,8 +269,8 @@ class AccountApi(Api):
             account_id=self.pk,
             access_token=self.access_token,
             callback_uri=self.get_callback_uri(
-                obj_name='preapproval', user_id=self.user.pk), 
-            callback=curry(Preapproval.objects.create_from_response, self), 
+                obj_name='preapproval', user_id=self.user.pk),
+            callback=curry(Preapproval.objects.create_from_response, self),
             **kwargs)
 
     def api_withdrawal_create(self, **kwargs):
@@ -267,8 +279,8 @@ class AccountApi(Api):
             account_id=self.pk,
             access_token=self.access_token,
             callback_uri=self.get_callback_uri(
-                obj_name='withdrawal', user_id=self.user.pk), 
-            callback=curry(Withdrawal.objects.create_from_response, self), 
+                obj_name='withdrawal', user_id=self.user.pk),
+            callback=curry(Withdrawal.objects.create_from_response, self),
             **kwargs)
 
     def api_subscription_plan_create(self, **kwargs):
@@ -277,23 +289,23 @@ class AccountApi(Api):
             account_id=self.pk,
             access_token=self.access_token,
             callback_uri=self.get_callback_uri(
-                obj_name='subscription_plan', user_id=self.user.pk), 
-            callback=curry(SubscriptionPlan.objects.create_from_response, self), 
+                obj_name='subscription_plan', user_id=self.user.pk),
+            callback=curry(SubscriptionPlan.objects.create_from_response, self),
             **kwargs)
 
     def api_checkout_find(self, **kwargs):
         return self.api.checkout.find(
-            account_id=self.pk, 
+            account_id=self.pk,
             access_token=self.access_token, **kwargs)
 
     def api_preapproval_find(self, **kwargs):
         return self.api.preapproval.find(
-            account_id=self.account_id, 
+            account_id=self.account_id,
             access_token=self.access_token, **kwargs)
 
     def api_withdrawal_find(self, **kwargs):
         return self.api.withdrawal.find(
-            account_id=self.pk, 
+            account_id=self.pk,
             access_token=self.access_token, **kwargs)
 
     def api_subscription_plan_find(self, **kwargs):
@@ -304,7 +316,7 @@ class AccountApi(Api):
     def api_subscription_plan_get_button(self, **kwargs):
         return self.api.subscription_plan.get_button(
             account_id=self.pk,
-            access_token=self.access_token, 
+            access_token=self.access_token,
             callback=self.instance_identity, **kwargs)
 
 
@@ -325,35 +337,35 @@ class CheckoutApi(Api):
     @cached_property
     def dispute_uri(self):
         return self.api_checkout()[1].get('dispute_uri', None)
-        
+
     def api_checkout(self, **kwargs):
         return self.api.checkout(
-            checkout_id=self.pk, 
-            access_token=self.access_token, 
+            checkout_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_checkout_cancel(self, **kwargs):
         return self.api.checkout.cancel(
-            checkout_id=self.pk, 
-            access_token=self.access_token, 
+            checkout_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_checkout_refund(self, **kwargs):
         return self.api.checkout.refund(
-            checkout_id=self.pk, 
-            access_token=self.access_token, 
+            checkout_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_checkout_capture(self, **kwargs):
         return self.api.checkout.capture(
-            checkout_id=self.pk, 
-            access_token=self.access_token, 
+            checkout_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_checkout_modify(self, **kwargs):
         return self.api.checkout.modify(
-            checkout_id=self.pk, 
-            access_token=self.access_token, 
+            checkout_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
 
@@ -361,7 +373,10 @@ class PreapprovalApi(Api):
 
     @cached_property
     def access_token(self):
-        return self.account.access_token
+        if self.account is not None:
+            return self.account.access_token
+        elif self.app is not None:
+            return self.app.access_token
 
     @cached_property
     def preapproval_uri(self):
@@ -382,31 +397,38 @@ class PreapprovalApi(Api):
 
     def api_preapproval(self, **kwargs):
         return self.api.preapproval(
-            preapproval_id=self.pk, 
-            access_token=self.access_token, 
+            preapproval_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_preapproval_cancel(self, **kwargs):
         return self.api.preapproval.cancel(
-            preapproval_id=self.pk, 
-            access_token=self.access_token, 
+            preapproval_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_preapproval_modify(self, **kwargs):
         return self.api.preapproval.modify(
-            preapproval_id=self.pk, 
-            access_token=self.access_token, 
+            preapproval_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
-    def api_checkout_create(self, **kwargs):
+    def api_checkout_create(self, account=None, **kwargs):
+        """Create a checkout using this preapproval. In case that preapproval
+        was authorized to send money to any account, account instance should be
+        supplied as a keyword argument.
+
+        """
+        assert bool(account) != bool(self.account), \
+            "You should supply account instance if preapproval was created on the App level"
         Checkout = get_wepay_model('checkout')
+        account = account or self.account
         return self.api.checkout.create(
-            account_id=self.account.pk,
-            access_token=self.access_token,
+            account_id=account.pk,
+            access_token=account.access_token,
             callback_uri=self.get_callback_uri(
-                obj_name='checkout', user_id=self.user.pk), 
-            callback=curry(Checkout.objects.create_from_response, self.account), 
-            **kwargs)
+                obj_name='checkout', user_id=account.user.pk),
+            callback=curry(Checkout.objects.create_from_response, account), **kwargs)
 
 
 class WithdrawalApi(Api):
@@ -430,37 +452,37 @@ class WithdrawalApi(Api):
 
     def api_withdrawal(self, **kwargs):
         return self.api.withdrawal(
-            withdrawal_id=self.pk, 
-            access_token=self.access_token, 
+            withdrawal_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_withdrawal_modify(self, **kwargs):
         return self.api.withdrawal.modify(
-            withdrawal_id=self.pk, 
-            access_token=self.access_token, 
+            withdrawal_id=self.pk,
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
-        
+
 
 class CreditCardApi(Api):
     def api_credit_card(self, **kwargs):
         return self.api.credit_card(
             client_id=self.app.client_id,
             client_secret=self.app.client_secret,
-            credit_card_id=self.pk, 
+            credit_card_id=self.pk,
             callback=self.instance_update, **kwargs)
 
     def api_credit_card_authorize(self, **kwargs):
         return self.api.credit_card.authorize(
             client_id=self.app.client_id,
             client_secret=self.app.client_secret,
-            credit_card_id=self.pk, 
+            credit_card_id=self.pk,
             callback=self.instance_update, **kwargs)
 
     def api_credit_card_delete(self, **kwargs):
         return self.api.credit_card.delete(
             client_id=self.app.client_id,
             client_secret=self.app.client_secret,
-            credit_card_id=self.pk, 
+            credit_card_id=self.pk,
             callback=self.instance_update, **kwargs)
 
 
@@ -469,7 +491,7 @@ class SubscriptionPlanApi(Api):
     @cached_property
     def access_token(self):
         return self.account.access_token
-    
+
     @cached_property
     def callback_uri(self):
         return self.api_subscription_plan()[1].get('callback_uri', None)
@@ -477,27 +499,27 @@ class SubscriptionPlanApi(Api):
     def api_subscription_plan(self, **kwargs):
         return self.api.subscription_plan(
             subscription_plan_id=self.pk,
-            access_token=self.access_token, 
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_subscription_plan_delete(self, **kwargs):
         return self.api.subscription_plan.delete(
             subscription_plan_id=self.pk,
-            access_token=self.access_token, 
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
-            
+
     def api_subscription_plan_get_button(self, **kwargs):
         return self.api.subscription_plan.get_button(
             account_id=self.account.pk,
             button_type='subscription_plan',
             subscription_plan_id=self.pk,
-            access_token=self.access_token, 
+            access_token=self.access_token,
             callback=self.instance_identity, **kwargs)
 
     def api_subscription_plan_modify(self, **kwargs):
         return self.api.subscription_plan.modify(
             subscription_plan_id=self.pk,
-            access_token=self.access_token, 
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_subscription_create(self, **kwargs):
@@ -520,7 +542,7 @@ class SubscriptionApi(Api):
     @cached_property
     def access_token(self):
         return self.subscription_plan.access_token
-    
+
     @cached_property
     def subscription_uri(self):
         return self.api_subscription()[1].get('subscription_uri', None)
@@ -536,19 +558,19 @@ class SubscriptionApi(Api):
     def api_subscription(self, **kwargs):
         return self.api.subscription(
             subscription_id=self.pk,
-            access_token=self.access_token, 
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_subscription_cancel(self, **kwargs):
         return self.api.subscription.cancel(
             subscription_id=self.pk,
-            access_token=self.access_token, 
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_subscription_modify(self, **kwargs):
         return self.api.subscription.modify(
             subscription_id=self.pk,
-            access_token=self.access_token, 
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_subscription_charge_find(self, **kwargs):
@@ -563,17 +585,15 @@ class SubscriptionChargeApi(Api):
     @cached_property
     def access_token(self):
         return self.subscription_plan.access_token
-    
+
     def api_subscription_charge(self, **kwargs):
         return self.api.subscription_charge(
             subscription_charge_id=self.pk,
-            access_token=self.access_token, 
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
 
     def api_subscription_charge_refund(self, **kwargs):
         return self.api.subscription_charge.refund(
             subscription_charge_id=self.pk,
-            access_token=self.access_token, 
+            access_token=self.access_token,
             callback=self.instance_update, **kwargs)
-
-    
\ No newline at end of file
